#include "club.h"

Club::Club(void)
: theta(0), aimPoint(0,0,0), swAngle(0)
{

	//  read in the bit map
	//  the file name is hard-coded
	BITMAPINFOHEADER	bitmapInfoHeader;	// bitmap info header
	bitmapData=LoadBitmapFile("club.bmp", &bitmapInfoHeader);


	// generate texture object
	unsigned int hTexture;
	glGenTextures(1,&hTexture);  

	// create the club
	theClub= gluNewQuadric();
	gluQuadricTexture(theClub, GL_TRUE);
	

	// create texture
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, hTexture);

	// the following two commands tell openGL how to interpolate texture
	// values between the sampled values in your texture bitmap
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	// the following two commands tell openGL how to extend the texture
	// when needed
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	// replace color of textured object
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	// now give it the texture map
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, bitmapInfoHeader.biWidth,
				 bitmapInfoHeader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, bitmapData);


	// initialize the club list

	// draw the club
	hClub = glGenLists(1);

	glNewList(hClub, GL_COMPILE); 
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, hTexture);
	
	glColor3f(1.0f,1.0f,1.0f);
	glPushMatrix();
	glTranslatef(1.0f, 4.0f, 0.0f);
	glRotatef(10.0f,0.0f,0.0f,1.0f);
	glTranslatef(0.0f,50.0f,0.0f);
	glRotatef(90.0f,1.0f,0.0f,0.0f);
	gluCylinder(theClub, 2.0, 1.0, 49.5, 16, 32);
	glPopMatrix();

  
	// draw the head
	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	
	// draw back face
	glPushMatrix();
	glTranslatef(0.0f, 0.0f, 1.0f);
	glBegin(GL_QUAD_STRIP);
	glColor3f(.9f,.9f,.9f);
	glVertex3f(1.0f, 5.0f, 0.0f);
	glVertex3f(9.0f, 5.0f, 0.0f);
	glVertex3f(0.0f, 4.0f, 0.0f);
	glVertex3f(10.0f, 4.0f, 0.0f);

	glColor3f(.4f, .4f, .4f);
	glVertex3f(1.0f, 0.0f, -0.5f);
	glVertex3f(9.0f, 0.0f, -0.5f);
	glEnd();
	glPopMatrix();


	// draw front face
	glPushMatrix();
	glTranslatef(0.0f, 0.0f, -1.0f);
	glBegin(GL_QUAD_STRIP);
	glColor3f(.9f,.9f,.9f);
	glVertex3f(1.0f, 5.0f, 0.0f);
	glVertex3f(9.0f, 5.0f, 0.0f);
	glVertex3f(0.0f, 4.0f, 0.0f);
	glVertex3f(10.0f, 4.0f, 0.0f);

	glColor3f(.5f, .5f, .5f);
	glVertex3f(1.0f, 0.0f, 0.1f);
	glVertex3f(9.0f, 0.0f, 0.1f);

	glEnd();
	glPopMatrix();



	// draw sides
	glBegin(GL_QUAD_STRIP);
	glColor3f(0.8f, 0.8f, 0.8f);
	glVertex3f(1.0f, 5.0f, -1.0f);
	glVertex3f(1.0f, 5.0f, 1.0f);
	glVertex3f(9.0f, 5.0f, -1.0f);
	glVertex3f(9.0f, 5.0f, 1.0f);

	glColor3f(0.9f, 0.9f, 0.9f);
	glVertex3f(10.0f, 4.0f, -1.0f);
	glVertex3f(10.0f, 4.0f, 1.0f);

	glColor3f(0.3f, 0.3f, 0.3f);
	glVertex3f(9.0f, 0.0f, -0.9f);
	glVertex3f(9.0f, 0.0f, 0.50f);
	glVertex3f(1.0f, 0.0f, -0.9f);
	glVertex3f(1.0f, 0.0f, 0.5f);

	glColor3f(0.9f, 0.9f, 0.9f);
	glVertex3f(0.0f, 4.0f, -1.0f);
	glVertex3f(0.0f, 4.0f, 1.0f);

	glColor3f(0.8f, 0.8f, 0.8f);
	glVertex3f(1.0f, 5.0f, -1.0f);
	glVertex3f(1.0f, 5.0f, 1.0f);
	glEnd();

	glPopMatrix();



	glEndList();

}

void Club::aimAt(Vector _aimPoint)
{
	aimPoint = _aimPoint;
}

void Club::setTheta(float _theta)
{
	theta = _theta;
}

void Club::setSwingAngle(float angle)
{
	swAngle = angle;
}

float Club::getSwingAngle()
{
	return swAngle;
}

void Club::draw() const
{
	glPushMatrix();
	// Translate to roughly the ball's position
	glTranslatef(aimPoint.x, aimPoint.y, aimPoint.z);
	// Rotate around y axis
	glRotatef(theta, 0, 1, 0);
	glTranslatef(-0.17, -.05, .1); // fine adjustment to point middle of club at ball
	// Rotate around top of club for swing angle
	glTranslatef(0,1.7,0);
	glRotatef(-swAngle, 1,0,0);
	glTranslatef(0,-1.7,0);
	// scale to a reasonable size 
	glScalef(0.03, 0.03, 0.03);
	glCallList(hClub);
	glPopMatrix();
}

Club::~Club(void)
{	
	glDeleteTextures(1, (const unsigned int*) &hTexture);
	gluDeleteQuadric(theClub);
	glDeleteLists(1,hClub);
}
