#include "hole.h"

// Amount the ball can be above or below the hole and still hit it
const float Hole::TOLERANCE = 0.08f;

Hole::Hole(const Vector& location_, float radius_)
: location(location_), radius(radius_)
{
}

Hole::~Hole(void)
{
}

void Hole::draw() const
{
	// draw it like a squashed sphere, which is easy and looks a lot like a disc
	glColor3f(0.0, 0.0, 0.0);
	glPushMatrix();
	glTranslatef(location.x, location.y, location.z); // move to center
	glScalef(radius, 0.01, radius); // scale radius in x and z, squash in y
	glutSolidSphere(1.0f,20,20);   // draw a unit sphere at the origin
	glPopMatrix(); //end sphere draw
}

// Check if point is in the hole
bool Hole::isIn(const Vector& point) const
{
	Vector displacement = point - location;

	return fabs(displacement.y) < TOLERANCE
		&& displacement.magSq() < radius*radius;
}