// camera.cpp - implementation of camera class
#include <iostream>
#include <GL/glut.h>
#include "camera.h"

using namespace std;

Camera::Camera()
: focalPoint(Vector(0,0,0)), r(1), theta(0), phi(0)
{
}

void Camera::lookAt(Vector newFocalPoint)
{
	focalPoint = newFocalPoint;
}

void Camera::doTransform() const
{
	glTranslatef(0, 0, -r);                                     // move r units back from focal point
	glRotatef(phi, 1, 0, 0);                                    // rotate phi degrees "up"
	glRotatef(theta, 0, 1, 0);                                  // rotate theta degrees "around"
	glTranslatef(-focalPoint.x, -focalPoint.y, -focalPoint.z);  // move to focal point
};

void Camera::setPosition(float newR, float newTheta, float newPhi)
{
	r = newR;
	theta = newTheta;
	phi = newPhi;
};

void Camera::setRelativePosition(float dr, float dTheta, float dPhi)
{
	r = r+dr;
	theta = theta + dTheta;
	phi = phi + dPhi;

	if(phi > 90)
		phi = 90;
	else if(phi < -90)
		phi = -90;

	if(r < 0)
		r = 0;
}

Camera::~Camera()
{
};
