#pragma once
#include <windows.h>
// We need windows.h for BITMAPINFOHEADER but it defines max, which we redefine later
#undef max

#include <math.h>
#include <iostream>
#include <gl/glut.h>
using namespace std;

unsigned char *LoadBitmapFile(char *filename, BITMAPINFOHEADER *bitmapInfoHeader);

////// Defines
#define BITMAP_ID 0x4D42		// the universal bitmap ID

class Triangle;

struct Vector
{
	Vector() {}
	Vector(float x_, float y_, float z_)
		: x(x_), y(y_), z(z_)
	{}

	Vector operator+ (const Vector& rhs) const { return Vector(x + rhs.x, y + rhs.y, z + rhs.z); }
	Vector operator* (float n) const { return Vector(x * n, y * n, z * n); }
	Vector operator- (const Vector& rhs) const { return *this + (rhs * -1.0f); }

	float dot(const Vector& rhs) const { return x*rhs.x + y*rhs.y + z*rhs.z; }
	Vector cross(const Vector& rhs) const
		{ return Vector(y*rhs.z - z*rhs.y, z*rhs.x - x*rhs.z, x*rhs.y - y*rhs.x); }
	float magSq() const { return dot(*this); } 

	Vector& normalize()
		{
			float mag = (float) sqrt(magSq());
			if(mag == 0)
				return *this;
			x /= mag;
			y /= mag;
			z /= mag;
			return *this;
		}

	float x,y,z;
};

ostream& operator<<(ostream& stream, Vector v);

struct Collision
{
	Collision() {};
	Collision(Vector where_, const Triangle* tri) //Vector normal_)
		: where(where_), triangle(tri)      //normal(normal_)
	{}

	Vector where;
//	Vector normal;
	const Triangle* triangle;
};

struct Color
{
	Color() {}
	Color(float r_, float g_, float b_)
		: r(r_), g(g_), b(b_)
	{}

	float r,g,b;
};;;

// returns maximum of two floats, since the max macro isn't working for some reason
float max(float a, float b);

// applies a transformation (encoded as a 16-element array representing a 4x4 matrix in
// column-major form, the format used by GL) to a point, returning the transformed point
Vector transformPt(const float transform[16], const Vector& point);